Class: Yerba::Document
- Inherits:
-
Object
- Object
- Yerba::Document
- Defined in:
- lib/yerba/document.rb,
ext/yerba/yerba.c
Constant Summary collapse
- ROOT_SELECTOR =
""
Class Method Summary collapse
- .cache ⇒ Object
- .clear_cache! ⇒ Object
- .from(object, path: nil) ⇒ Object
-
.parse(content) ⇒ Object
Document.parse(content).
Instance Method Summary collapse
- #<<(item) ⇒ Object
-
#[](path) ⇒ Object
document[](path) → Yerba::Scalar, Yerba::Map, Yerba::Sequence, Array, or nil.
- #[]=(key, value) ⇒ Object
- #apply(yerbafile = nil) ⇒ Object
- #apply!(yerbafile = nil) ⇒ Object
-
#apply_yerbafile(*args) ⇒ Object
document.apply_yerbafile(yerbafile_path = nil).
-
#blank_lines(path, count) ⇒ Object
document.blank_lines(path, count).
-
#changed? ⇒ Boolean
document.changed?.
- #concat(items) ⇒ Object
-
#condition?(*args) ⇒ Boolean
document.condition?(condition, path: “”).
-
#delete(*args) ⇒ Object
document.delete(path, condition: nil).
- #dig(*keys) ⇒ Object
-
#exists?(path) ⇒ Boolean
document.exists?(path).
- #fetch(selector) ⇒ Object
-
#find(*args) ⇒ Object
document.find(path, condition: nil, select: nil).
- #find_by ⇒ Object
-
#get_collection_style(path) ⇒ Object
document.get_collection_style(path) → :flow, :block, or nil.
-
#get_quote_style(path) ⇒ Object
document.get_quote_style(path) → :plain, :single, :double, :literal, etc.
-
#get_sequence_indent(path) ⇒ Object
document.get_sequence_indent(path) → :compact, :indented, or nil.
-
#initialize(*args) ⇒ Object
constructor
Document.new(path).
-
#insert(*args) ⇒ Object
document.insert(path, value, before: nil, after: nil, at: nil).
-
#insert_object(*args) ⇒ Object
document.insert_object(path, object, before: nil, after: nil, at: nil).
-
#insert_objects(path, array) ⇒ Object
document.insert_objects(path, array).
- #inspect ⇒ Object
-
#location(*args) ⇒ Object
document.location(selector = nil) → Yerba::Location.
-
#locations(path) ⇒ Object
document.locations(selector) → [Yerba::Location, …].
- #map? ⇒ Boolean
-
#node_at(path) ⇒ Object
document[](path) → Yerba::Scalar, Yerba::Map, Yerba::Sequence, Array, or nil.
-
#path ⇒ Object
document.path.
- #pluck ⇒ Object
-
#quote_style(*args) ⇒ Object
document.quote_style(path: nil, key_style: nil, value_style: nil).
-
#remove(path, value) ⇒ Object
document.remove(path, value).
-
#remove_at(path, index) ⇒ Object
document.remove_at(path, index).
-
#rename(source, destination) ⇒ Object
document.rename(source, destination).
-
#replace_content!(content) ⇒ Object
document.replace_content!(content) — re-parse from YAML string, keeping the same Ruby object.
-
#resolve_selectors(path) ⇒ Object
document.resolve_selectors(path) → [“[0].speakers”, “[0].speakers”, …].
- #root ⇒ Object
- #root=(value) ⇒ Object
- #save!(apply: false) ⇒ Object
- #save_to!(path) ⇒ Object
- #selector ⇒ Object
-
#selectors ⇒ Object
document.selectors → [“database”, “database.host”, “tags”, “tags[]”, …].
- #sequence? ⇒ Boolean
-
#set(*args) ⇒ Object
document.set(path, value, condition: nil, if_exists: false, if_missing: false).
-
#set_collection_style(path, style) ⇒ Object
document.set_collection_style(path, style).
-
#set_quote_style(path, style) ⇒ Object
document.set_quote_style(path, style).
-
#set_sequence_indent(path, style) ⇒ Object
document.set_sequence_indent(path, style).
-
#sort(*args) ⇒ Object
document.sort(path = “”, by: nil, order: nil, case_sensitive: false).
-
#sort_keys(path, order) ⇒ Object
document.sort_keys(path, order).
- #to_a ⇒ Object
- #to_h ⇒ Object
-
#to_s ⇒ Object
document.to_s.
- #to_yaml ⇒ Object
- #valid?(schema, selector: nil) ⇒ Boolean
-
#valid_selector?(path) ⇒ Boolean
document.valid_selector?(selector) → true/false.
- #validate(schema, selector: nil) ⇒ Object
-
#validate_schema(*args) ⇒ Object
document.validate_schema(schema_json, selector = nil).
- #validate_selector!(selector) ⇒ Object
-
#value_at(path) ⇒ Object
document.get_value(path) → parsed Ruby object (Hash/Array/String/Integer/etc).
- #where ⇒ Object
-
#write! ⇒ Object
document.save!.
Constructor Details
#initialize(*args) ⇒ Object
Document.new(path)
113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 |
# File 'ext/yerba/yerba.c', line 113
static VALUE document_initialize(int argc, VALUE *argv, VALUE self) {
VALUE path;
rb_scan_args(argc, argv, "01", &path);
YerbaParseResult result;
if (NIL_P(path)) {
result = yerba_document_parse("---\n");
rb_iv_set(self, "@path", Qnil);
} else {
const char *file_path = StringValueCStr(path);
result = yerba_document_parse_file(file_path);
rb_iv_set(self, "@path", path);
}
if (!result.document) {
VALUE message = make_utf8_string(result.error);
yerba_string_free(result.error);
rb_raise(rb_eParseError, "%s", StringValueCStr(message));
}
RTYPEDDATA_DATA(self) = result.document;
return self;
}
|
Class Method Details
.cache ⇒ Object
7 8 9 |
# File 'lib/yerba/document.rb', line 7 def self.cache @cache ||= Hash.new { |hash, path| hash[path] = new(path) } end |
.clear_cache! ⇒ Object
11 12 13 |
# File 'lib/yerba/document.rb', line 11 def self.clear_cache! @cache = nil end |
.from(object, path: nil) ⇒ Object
15 16 17 18 19 |
# File 'lib/yerba/document.rb', line 15 def self.from(object, path: nil) document = parse(Formatting.to_yaml_document(object)) document.instance_variable_set(:@path, path) if path document end |
.parse(content) ⇒ Object
Document.parse(content)
164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 |
# File 'ext/yerba/yerba.c', line 164
static VALUE document_s_parse(VALUE klass, VALUE content) {
const char *yaml_content = StringValueCStr(content);
YerbaParseResult result = yerba_document_parse(yaml_content);
if (!result.document) {
VALUE message = make_utf8_string(result.error);
yerba_string_free(result.error);
rb_raise(rb_eParseError, "%s", StringValueCStr(message));
}
VALUE instance = document_alloc(klass);
RTYPEDDATA_DATA(instance) = result.document;
rb_iv_set(instance, "@path", Qnil);
return instance;
}
|
Instance Method Details
#<<(item) ⇒ Object
92 93 94 95 96 97 98 99 100 101 102 |
# File 'lib/yerba/document.rb', line 92 def <<(item) if root.is_a?(Scalar) raise Error, "document root is not set. Use `document.root = []` first" end if root.is_a?(Map) raise Error, "document root is a Map, not a Sequence. Use `document[\"key\"] = value` to set keys, or `document.root = []` to switch to a Sequence" end root << item end |
#[](path) ⇒ Object
document[](path) → Yerba::Scalar, Yerba::Map, Yerba::Sequence, Array, or nil
246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 |
# File 'ext/yerba/yerba.c', line 246
static VALUE document_bracket(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char index_buffer[32];
if (RB_TYPE_P(path, T_FIXNUM)) {
snprintf(index_buffer, sizeof(index_buffer), "[%ld]", FIX2LONG(path));
path = rb_str_new_cstr(index_buffer);
}
const char *selector = StringValueCStr(path);
/* Wildcard: resolve to concrete selectors and return array of nodes */
if (strstr(selector, "[]") != NULL) {
char *json = yerba_document_resolve_selectors(document, selector);
if (!json) return rb_ary_new();
VALUE json_string = make_utf8_string(json);
yerba_string_free(json);
VALUE resolved = rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
long length = RARRAY_LEN(resolved);
VALUE array = rb_ary_new_capa(length);
for (long i = 0; i < length; i++) {
VALUE concrete_path = rb_ary_entry(resolved, i);
YerbaGetResult result = yerba_document_get(document, StringValueCStr(concrete_path));
VALUE node = node_from_get_result(result, self, concrete_path);
if (!NIL_P(node)) rb_ary_push(array, node);
}
return array;
}
YerbaGetResult result = yerba_document_get(document, selector);
return node_from_get_result(result, self, path);
}
|
#[]=(key, value) ⇒ Object
33 34 35 36 37 38 39 40 41 42 43 |
# File 'lib/yerba/document.rb', line 33 def []=(key, value) if root.is_a?(Scalar) raise Error, "document root is not set. Use `document.root = {}` or `document.root = []` first" end if root.is_a?(Sequence) raise Error, "document root is a Sequence, not a Map. Use `document << item` to append, or `document.root = {}` to switch to a Map" end root[key] = value end |
#apply(yerbafile = nil) ⇒ Object
122 123 124 125 126 |
# File 'lib/yerba/document.rb', line 122 def apply(yerbafile = nil) Yerbafile.apply!(self, yerbafile) self end |
#apply!(yerbafile = nil) ⇒ Object
115 116 117 118 119 120 |
# File 'lib/yerba/document.rb', line 115 def apply!(yerbafile = nil) apply(yerbafile) write! if changed? self end |
#apply_yerbafile(*args) ⇒ Object
document.apply_yerbafile(yerbafile_path = nil)
845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 |
# File 'ext/yerba/yerba.c', line 845
static VALUE document_apply_yerbafile(int argc, VALUE *argv, VALUE self) {
VALUE yerbafile_path;
rb_scan_args(argc, argv, "01", &yerbafile_path);
struct Document *document = get_document(self);
VALUE file_path = rb_iv_get(self, "@path");
const char *file_path_str = NIL_P(file_path) ? "" : StringValueCStr(file_path);
const char *yerbafile_path_str = NIL_P(yerbafile_path) ? NULL : StringValueCStr(yerbafile_path);
YerbaResult result = yerba_document_apply_yerbafile(document, file_path_str, yerbafile_path_str);
check_result(result);
return self;
}
|
#blank_lines(path, count) ⇒ Object
document.blank_lines(path, count)
835 836 837 838 839 840 841 842 |
# File 'ext/yerba/yerba.c', line 835
static VALUE document_blank_lines(VALUE self, VALUE path, VALUE count) {
struct Document *document = get_document(self);
YerbaResult result = yerba_document_blank_lines(document, StringValueCStr(path), NUM2SIZET(count));
check_result(result);
return self;
}
|
#changed? ⇒ Boolean
document.changed?
906 907 908 909 910 911 912 913 914 |
# File 'ext/yerba/yerba.c', line 906
static VALUE document_changed_p(VALUE self) {
VALUE path = rb_iv_get(self, "@path");
if (NIL_P(path)) return Qtrue;
VALUE current = document_to_s(self);
VALUE original = rb_funcall(rb_cFile, rb_intern("read"), 1, path);
return rb_str_equal(current, original) ? Qfalse : Qtrue;
}
|
#concat(items) ⇒ Object
104 105 106 |
# File 'lib/yerba/document.rb', line 104 def concat(items) root.concat(items) end |
#condition?(*args) ⇒ Boolean
document.condition?(condition, path: “”)
481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 |
# File 'ext/yerba/yerba.c', line 481
static VALUE document_condition_p(int argc, VALUE *argv, VALUE self) {
VALUE condition, opts;
rb_scan_args(argc, argv, "1:", &condition, &opts);
const char *parent_path = "";
if (!NIL_P(opts)) {
VALUE v_path = rb_hash_aref(opts, ID2SYM(rb_intern("path")));
if (!NIL_P(v_path)) {
parent_path = StringValueCStr(v_path);
}
}
struct Document *document = get_document(self);
return yerba_document_evaluate_condition(document, parent_path, StringValueCStr(condition)) ? Qtrue : Qfalse;
}
|
#delete(*args) ⇒ Object
document.delete(path, condition: nil)
662 663 664 665 666 667 668 669 670 671 672 673 674 |
# File 'ext/yerba/yerba.c', line 662
static VALUE document_delete(int argc, VALUE *argv, VALUE self) {
VALUE path, opts;
rb_scan_args(argc, argv, "1:", &path, &opts);
struct Document *document = get_document(self);
if (!should_proceed(document, opts)) return self;
YerbaResult result = yerba_document_delete(document, StringValueCStr(path));
check_result(result);
return self;
}
|
#dig(*keys) ⇒ Object
70 71 72 |
# File 'lib/yerba/document.rb', line 70 def dig(*keys) keys.reduce(self) { |node, key| node.nil? ? nil : node[key] } end |
#exists?(path) ⇒ Boolean
document.exists?(path)
287 288 289 290 291 |
# File 'ext/yerba/yerba.c', line 287
static VALUE document_exists_p(VALUE self, VALUE path) {
struct Document *document = get_document(self);
return yerba_document_exists(document, StringValueCStr(path)) ? Qtrue : Qfalse;
}
|
#fetch(selector) ⇒ Object
74 75 76 77 78 |
# File 'lib/yerba/document.rb', line 74 def fetch(selector) validate_selector!(selector) self[selector] end |
#find(*args) ⇒ Object
document.find(path, condition: nil, select: nil)
500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 |
# File 'ext/yerba/yerba.c', line 500
static VALUE document_find(int argc, VALUE *argv, VALUE self) {
VALUE path, opts;
rb_scan_args(argc, argv, "1:", &path, &opts);
const char *condition = NULL;
const char *select = NULL;
if (!NIL_P(opts)) {
VALUE v_condition = rb_hash_aref(opts, ID2SYM(rb_intern("condition")));
VALUE v_select = rb_hash_aref(opts, ID2SYM(rb_intern("select")));
if (!NIL_P(v_condition)) condition = StringValueCStr(v_condition);
if (!NIL_P(v_select)) select = StringValueCStr(v_select);
}
struct Document *document = get_document(self);
char *json = yerba_document_find(document, StringValueCStr(path), condition, select);
if (!json) return rb_ary_new();
VALUE json_string = make_utf8_string(json);
yerba_string_free(json);
return rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
}
|
#find_by ⇒ Object
80 81 82 |
# File 'lib/yerba/document.rb', line 80 def find_by(...) root.find_by(...) end |
#get_collection_style(path) ⇒ Object
document.get_collection_style(path) → :flow, :block, or nil
411 412 413 414 415 416 417 418 419 420 421 422 |
# File 'ext/yerba/yerba.c', line 411
static VALUE document_get_collection_style(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char *style = yerba_document_get_collection_style(document, StringValueCStr(path));
if (style == NULL) return Qnil;
VALUE symbol = ID2SYM(rb_intern(style));
yerba_string_free(style);
return symbol;
}
|
#get_quote_style(path) ⇒ Object
document.get_quote_style(path) → :plain, :single, :double, :literal, etc. or nil
376 377 378 379 380 381 382 383 384 385 386 387 |
# File 'ext/yerba/yerba.c', line 376
static VALUE document_get_quote_style(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char *style = yerba_document_get_quote_style(document, StringValueCStr(path));
if (style == NULL) return Qnil;
VALUE symbol = ID2SYM(rb_intern(style));
yerba_string_free(style);
return symbol;
}
|
#get_sequence_indent(path) ⇒ Object
document.get_sequence_indent(path) → :compact, :indented, or nil
446 447 448 449 450 451 452 453 454 455 456 457 |
# File 'ext/yerba/yerba.c', line 446
static VALUE document_get_sequence_indent(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char *style = yerba_document_get_sequence_indent(document, StringValueCStr(path));
if (style == NULL) return Qnil;
VALUE symbol = ID2SYM(rb_intern(style));
yerba_string_free(style);
return symbol;
}
|
#insert(*args) ⇒ Object
document.insert(path, value, before: nil, after: nil, at: nil)
595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 |
# File 'ext/yerba/yerba.c', line 595
static VALUE document_insert(int argc, VALUE *argv, VALUE self) {
VALUE path, value, opts;
rb_scan_args(argc, argv, "2:", &path, &value, &opts);
const char *before = NULL;
const char *after = NULL;
long long at = -1;
if (!NIL_P(opts)) {
VALUE v_before = rb_hash_aref(opts, ID2SYM(rb_intern("before")));
VALUE v_after = rb_hash_aref(opts, ID2SYM(rb_intern("after")));
VALUE v_at = rb_hash_aref(opts, ID2SYM(rb_intern("at")));
if (!NIL_P(v_before)) before = StringValueCStr(v_before);
if (!NIL_P(v_after)) after = StringValueCStr(v_after);
if (!NIL_P(v_at)) at = NUM2LL(v_at);
}
char number_buffer[64];
struct TypedValue typed_value = ruby_to_typed_value(value, number_buffer);
struct Document *document = get_document(self);
YerbaResult result = yerba_document_insert(document, StringValueCStr(path), typed_value.text, typed_value.type, before, after, at);
check_result(result);
return self;
}
|
#insert_object(*args) ⇒ Object
document.insert_object(path, object, before: nil, after: nil, at: nil)
624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 |
# File 'ext/yerba/yerba.c', line 624
static VALUE document_insert_object(int argc, VALUE *argv, VALUE self) {
VALUE path, object, opts;
rb_scan_args(argc, argv, "2:", &path, &object, &opts);
const char *before = NULL;
const char *after = NULL;
long long at = -1;
if (!NIL_P(opts)) {
VALUE v_before = rb_hash_aref(opts, ID2SYM(rb_intern("before")));
VALUE v_after = rb_hash_aref(opts, ID2SYM(rb_intern("after")));
VALUE v_at = rb_hash_aref(opts, ID2SYM(rb_intern("at")));
if (!NIL_P(v_before)) before = StringValueCStr(v_before);
if (!NIL_P(v_after)) after = StringValueCStr(v_after);
if (!NIL_P(v_at)) at = NUM2LL(v_at);
}
struct Document *document = get_document(self);
VALUE json_string = rb_funcall(rb_path2class("JSON"), rb_intern("generate"), 1, object);
YerbaResult result = yerba_document_insert_object(document, StringValueCStr(path), StringValueCStr(json_string), before, after, at);
check_result(result);
return self;
}
|
#insert_objects(path, array) ⇒ Object
document.insert_objects(path, array)
651 652 653 654 655 656 657 658 659 |
# File 'ext/yerba/yerba.c', line 651
static VALUE document_insert_objects(VALUE self, VALUE path, VALUE array) {
struct Document *document = get_document(self);
VALUE json_string = rb_funcall(rb_path2class("JSON"), rb_intern("generate"), 1, array);
YerbaResult result = yerba_document_insert_objects(document, StringValueCStr(path), StringValueCStr(json_string));
check_result(result);
return self;
}
|
#inspect ⇒ Object
160 161 162 163 164 165 166 |
# File 'lib/yerba/document.rb', line 160 def inspect if path "#<Yerba::Document path=#{path.inspect}>" else "#<Yerba::Document (parsed)>" end end |
#location(*args) ⇒ Object
document.location(selector = nil) → Yerba::Location
917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 |
# File 'ext/yerba/yerba.c', line 917
static VALUE document_location(int argc, VALUE *argv, VALUE self) {
VALUE path;
rb_scan_args(argc, argv, "01", &path);
struct Document *document = get_document(self);
const char *selector = NIL_P(path) ? "" : StringValueCStr(path);
YerbaGetResult result = yerba_document_get(document, selector);
if (result.error) {
yerba_get_result_free(result);
return Qnil;
}
if (result.location.start_line == 0 && result.location.end_line == 0 && strlen(selector) > 0) {
yerba_get_result_free(result);
return Qnil;
}
VALUE location = location_to_ruby(result.location);
yerba_get_result_free(result);
return location;
}
|
#locations(path) ⇒ Object
document.locations(selector) → [Yerba::Location, …]
327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 |
# File 'ext/yerba/yerba.c', line 327
static VALUE document_locations(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char *json = yerba_document_resolve_selectors(document, StringValueCStr(path));
if (!json) return rb_ary_new();
VALUE json_string = make_utf8_string(json);
yerba_string_free(json);
VALUE resolved = rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
long length = RARRAY_LEN(resolved);
VALUE array = rb_ary_new_capa(length);
for (long i = 0; i < length; i++) {
VALUE concrete_path = rb_ary_entry(resolved, i);
YerbaGetResult result = yerba_document_get(document, StringValueCStr(concrete_path));
if (result.error) {
yerba_get_result_free(result);
continue;
}
if (result.location.start_line == 0 && result.location.end_line == 0) {
yerba_get_result_free(result);
continue;
}
VALUE location = location_to_ruby(result.location);
yerba_get_result_free(result);
rb_ary_push(array, location);
}
return array;
}
|
#node_at(path) ⇒ Object
document[](path) → Yerba::Scalar, Yerba::Map, Yerba::Sequence, Array, or nil
246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 |
# File 'ext/yerba/yerba.c', line 246
static VALUE document_bracket(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char index_buffer[32];
if (RB_TYPE_P(path, T_FIXNUM)) {
snprintf(index_buffer, sizeof(index_buffer), "[%ld]", FIX2LONG(path));
path = rb_str_new_cstr(index_buffer);
}
const char *selector = StringValueCStr(path);
/* Wildcard: resolve to concrete selectors and return array of nodes */
if (strstr(selector, "[]") != NULL) {
char *json = yerba_document_resolve_selectors(document, selector);
if (!json) return rb_ary_new();
VALUE json_string = make_utf8_string(json);
yerba_string_free(json);
VALUE resolved = rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
long length = RARRAY_LEN(resolved);
VALUE array = rb_ary_new_capa(length);
for (long i = 0; i < length; i++) {
VALUE concrete_path = rb_ary_entry(resolved, i);
YerbaGetResult result = yerba_document_get(document, StringValueCStr(concrete_path));
VALUE node = node_from_get_result(result, self, concrete_path);
if (!NIL_P(node)) rb_ary_push(array, node);
}
return array;
}
YerbaGetResult result = yerba_document_get(document, selector);
return node_from_get_result(result, self, path);
}
|
#path ⇒ Object
document.path
943 944 945 |
# File 'ext/yerba/yerba.c', line 943 static VALUE document_path(VALUE self) { return rb_iv_get(self, "@path"); } |
#pluck ⇒ Object
88 89 90 |
# File 'lib/yerba/document.rb', line 88 def pluck(...) root.pluck(...) end |
#quote_style(*args) ⇒ Object
document.quote_style(path: nil, key_style: nil, value_style: nil)
808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 |
# File 'ext/yerba/yerba.c', line 808
static VALUE document_quote_style(int argc, VALUE *argv, VALUE self) {
VALUE opts;
rb_scan_args(argc, argv, ":", &opts);
const char *path = NULL;
const char *key_style = NULL;
const char *value_style = NULL;
if (!NIL_P(opts)) {
VALUE v_path = rb_hash_aref(opts, ID2SYM(rb_intern("path")));
VALUE v_key_style = rb_hash_aref(opts, ID2SYM(rb_intern("key_style")));
VALUE v_value_style = rb_hash_aref(opts, ID2SYM(rb_intern("value_style")));
if (!NIL_P(v_path)) path = StringValueCStr(v_path);
if (!NIL_P(v_key_style)) key_style = StringValueCStr(v_key_style);
if (!NIL_P(v_value_style)) value_style = StringValueCStr(v_value_style);
}
struct Document *document = get_document(self);
YerbaResult result = yerba_document_quote_style(document, path, key_style, value_style);
check_result(result);
return self;
}
|
#remove(path, value) ⇒ Object
document.remove(path, value)
677 678 679 680 681 682 683 684 |
# File 'ext/yerba/yerba.c', line 677
static VALUE document_remove(VALUE self, VALUE path, VALUE value) {
struct Document *document = get_document(self);
YerbaResult result = yerba_document_remove(document, StringValueCStr(path), StringValueCStr(value));
check_result(result);
return self;
}
|
#remove_at(path, index) ⇒ Object
document.remove_at(path, index)
687 688 689 690 691 692 693 694 |
# File 'ext/yerba/yerba.c', line 687
static VALUE document_remove_at(VALUE self, VALUE path, VALUE index) {
struct Document *document = get_document(self);
YerbaResult result = yerba_document_remove_at(document, StringValueCStr(path), NUM2SIZET(index));
check_result(result);
return self;
}
|
#rename(source, destination) ⇒ Object
document.rename(source, destination)
697 698 699 700 701 702 703 704 |
# File 'ext/yerba/yerba.c', line 697
static VALUE document_rename(VALUE self, VALUE source, VALUE destination) {
struct Document *document = get_document(self);
YerbaResult result = yerba_document_rename(document, StringValueCStr(source), StringValueCStr(destination));
check_result(result);
return self;
}
|
#replace_content!(content) ⇒ Object
document.replace_content!(content) — re-parse from YAML string, keeping the same Ruby object
141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 |
# File 'ext/yerba/yerba.c', line 141
static VALUE document_replace_content(VALUE self, VALUE content) {
const char *yaml_content = StringValueCStr(content);
YerbaParseResult result = yerba_document_parse(yaml_content);
if (!result.document) {
VALUE message = make_utf8_string(result.error);
yerba_string_free(result.error);
rb_raise(rb_eParseError, "%s", StringValueCStr(message));
}
struct Document *old_document = get_document(self);
if (old_document) {
yerba_document_free(old_document);
}
RTYPEDDATA_DATA(self) = result.document;
return self;
}
|
#resolve_selectors(path) ⇒ Object
document.resolve_selectors(path) → [“[0].speakers”, “[0].speakers”, …]
363 364 365 366 367 368 369 370 371 372 373 |
# File 'ext/yerba/yerba.c', line 363
static VALUE document_resolve_selectors(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char *json = yerba_document_resolve_selectors(document, StringValueCStr(path));
if (!json) return rb_ary_new();
VALUE json_string = make_utf8_string(json);
yerba_string_free(json);
return rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
}
|
#root ⇒ Object
25 26 27 |
# File 'lib/yerba/document.rb', line 25 def root self[ROOT_SELECTOR] end |
#root=(value) ⇒ Object
29 30 31 |
# File 'lib/yerba/document.rb', line 29 def root=(value) replace_content!(Formatting.to_yaml_document(value)) end |
#save!(apply: false) ⇒ Object
108 109 110 111 112 113 |
# File 'lib/yerba/document.rb', line 108 def save!(apply: false) Yerbafile.apply!(self, apply) if apply write! self end |
#save_to!(path) ⇒ Object
65 66 67 68 |
# File 'lib/yerba/document.rb', line 65 def save_to!(path) @path = path save! end |
#selector ⇒ Object
21 22 23 |
# File 'lib/yerba/document.rb', line 21 def selector ROOT_SELECTOR end |
#selectors ⇒ Object
document.selectors → [“database”, “database.host”, “tags”, “tags[]”, …]
314 315 316 317 318 319 320 321 322 323 324 |
# File 'ext/yerba/yerba.c', line 314
static VALUE document_selectors(VALUE self) {
struct Document *document = get_document(self);
char *json = yerba_document_selectors(document);
if (!json) return rb_ary_new();
VALUE json_string = make_utf8_string(json);
yerba_string_free(json);
return rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
}
|
#sequence? ⇒ Boolean
49 50 51 |
# File 'lib/yerba/document.rb', line 49 def sequence? root.is_a?(Sequence) end |
#set(*args) ⇒ Object
document.set(path, value, condition: nil, if_exists: false, if_missing: false)
562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 |
# File 'ext/yerba/yerba.c', line 562
static VALUE document_set(int argc, VALUE *argv, VALUE self) {
VALUE path, value, opts;
rb_scan_args(argc, argv, "2:", &path, &value, &opts);
struct Document *document = get_document(self);
if (!should_proceed(document, opts)) return self;
if (!NIL_P(opts)) {
VALUE v_if_exists = rb_hash_aref(opts, ID2SYM(rb_intern("if_exists")));
VALUE v_if_missing = rb_hash_aref(opts, ID2SYM(rb_intern("if_missing")));
if (RTEST(v_if_exists) && !yerba_document_exists(document, StringValueCStr(path))) return self;
if (RTEST(v_if_missing) && yerba_document_exists(document, StringValueCStr(path))) return self;
}
char number_buffer[64];
struct TypedValue typed_value = ruby_to_typed_value(value, number_buffer);
bool all = false;
if (!NIL_P(opts)) {
VALUE v_all = rb_hash_aref(opts, ID2SYM(rb_intern("all")));
if (RTEST(v_all)) all = true;
}
YerbaResult result = yerba_document_set(document, StringValueCStr(path), typed_value.text, typed_value.type, all);
check_result(result);
return self;
}
|
#set_collection_style(path, style) ⇒ Object
document.set_collection_style(path, style)
425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 |
# File 'ext/yerba/yerba.c', line 425
static VALUE document_set_collection_style(VALUE self, VALUE path, VALUE style) {
struct Document *document = get_document(self);
const char *style_string;
if (RB_TYPE_P(style, T_SYMBOL)) {
style_string = rb_id2name(SYM2ID(style));
} else if (RB_TYPE_P(style, T_STRING)) {
style_string = StringValueCStr(style);
} else {
rb_raise(rb_eError, "Invalid collection style (expected Symbol or String)");
}
YerbaResult result = yerba_document_set_collection_style(document, StringValueCStr(path), style_string);
check_result(result);
return self;
}
|
#set_quote_style(path, style) ⇒ Object
document.set_quote_style(path, style)
390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 |
# File 'ext/yerba/yerba.c', line 390
static VALUE document_set_quote_style(VALUE self, VALUE path, VALUE style) {
struct Document *document = get_document(self);
const char *style_string;
if (RB_TYPE_P(style, T_SYMBOL)) {
style_string = rb_id2name(SYM2ID(style));
} else if (RB_TYPE_P(style, T_STRING)) {
style_string = StringValueCStr(style);
} else {
rb_raise(rb_eError, "Invalid quote style (expected Symbol or String)");
}
YerbaResult result = yerba_document_set_quote_style(document, StringValueCStr(path), style_string);
check_result(result);
return self;
}
|
#set_sequence_indent(path, style) ⇒ Object
document.set_sequence_indent(path, style)
460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 |
# File 'ext/yerba/yerba.c', line 460
static VALUE document_set_sequence_indent(VALUE self, VALUE path, VALUE style) {
struct Document *document = get_document(self);
const char *style_string;
if (RB_TYPE_P(style, T_SYMBOL)) {
style_string = rb_id2name(SYM2ID(style));
} else if (RB_TYPE_P(style, T_STRING)) {
style_string = StringValueCStr(style);
} else {
rb_raise(rb_eError, "Invalid sequence indent style (expected Symbol or String)");
}
YerbaResult result = yerba_document_set_sequence_indent(document, StringValueCStr(path), style_string);
check_result(result);
return self;
}
|
#sort(*args) ⇒ Object
document.sort(path = “”, by: nil, order: nil, case_sensitive: false)
707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 760 761 762 763 764 765 766 767 768 769 770 771 772 773 774 775 776 777 778 779 780 781 782 783 784 785 786 787 |
# File 'ext/yerba/yerba.c', line 707
static VALUE document_sort(int argc, VALUE *argv, VALUE self) {
VALUE path, opts;
rb_scan_args(argc, argv, "01:", &path, &opts);
if (NIL_P(path)) path = rb_str_new_cstr("");
const char *by = NULL;
bool case_sensitive = false;
VALUE v_order = Qnil;
if (!NIL_P(opts)) {
VALUE v_by = rb_hash_aref(opts, ID2SYM(rb_intern("by")));
v_order = rb_hash_aref(opts, ID2SYM(rb_intern("order")));
VALUE v_case_sensitive = rb_hash_aref(opts, ID2SYM(rb_intern("case_sensitive")));
if (SYMBOL_P(v_by)) {
VALUE by_string = rb_sym2str(v_by);
by = StringValueCStr(by_string);
} else if (!NIL_P(v_by)) {
by = StringValueCStr(v_by);
}
if (RTEST(v_case_sensitive)) {
case_sensitive = true;
}
}
struct Document *document = get_document(self);
const char *path_string = StringValueCStr(path);
if (RB_TYPE_P(v_order, T_ARRAY)) {
VALUE order_csv = rb_ary_join(v_order, rb_str_new_cstr(","));
const char *order_string = StringValueCStr(order_csv);
const char *reorder_path = StringValueCStr(path);
const char *reorder_by;
if (by) {
VALUE reorder_by_value = rb_hash_aref(opts, ID2SYM(rb_intern("by")));
if (SYMBOL_P(reorder_by_value)) {
reorder_by_value = rb_sym2str(reorder_by_value);
}
reorder_by = StringValueCStr(reorder_by_value);
} else {
reorder_by = ".";
}
YerbaResult result = yerba_document_reorder(document, reorder_path, reorder_by, order_string);
check_result(result);
return self;
}
const char *order = NULL;
if (SYMBOL_P(v_order)) {
VALUE order_string = rb_sym2str(v_order);
order = StringValueCStr(order_string);
} else if (!NIL_P(v_order)) {
order = StringValueCStr(v_order);
}
VALUE by_with_order = Qnil;
if (order && strcmp(order, "desc") == 0) {
if (by) {
by_with_order = rb_sprintf("%s:desc", by);
} else {
by_with_order = rb_str_new_cstr(":desc");
}
by = StringValueCStr(by_with_order);
}
YerbaResult result = yerba_document_sort(document, path_string, by, case_sensitive);
check_result(result);
return self;
}
|
#sort_keys(path, order) ⇒ Object
document.sort_keys(path, order)
790 791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 |
# File 'ext/yerba/yerba.c', line 790
static VALUE document_sort_keys(VALUE self, VALUE path, VALUE order) {
struct Document *document = get_document(self);
VALUE order_string;
if (RB_TYPE_P(order, T_ARRAY)) {
order_string = rb_ary_join(order, rb_str_new_cstr(","));
} else {
order_string = order;
}
YerbaResult result = yerba_document_sort_keys(document, StringValueCStr(path), StringValueCStr(order_string));
check_result(result);
return self;
}
|
#to_a ⇒ Object
57 58 59 |
# File 'lib/yerba/document.rb', line 57 def to_a value_at(ROOT_SELECTOR) end |
#to_h ⇒ Object
53 54 55 |
# File 'lib/yerba/document.rb', line 53 def to_h value_at(ROOT_SELECTOR) end |
#to_s ⇒ Object
document.to_s
880 881 882 883 884 885 886 887 888 |
# File 'ext/yerba/yerba.c', line 880 static VALUE document_to_s(VALUE self) { struct Document *document = get_document(self); char *content = yerba_document_to_string(document); VALUE string = make_utf8_string(content); yerba_string_free(content); return string; } |
#to_yaml ⇒ Object
61 62 63 |
# File 'lib/yerba/document.rb', line 61 def to_yaml to_s end |
#valid?(schema, selector: nil) ⇒ Boolean
128 129 130 131 132 |
# File 'lib/yerba/document.rb', line 128 def valid?(schema, selector: nil) errors = validate(schema, selector: selector) errors.empty? end |
#valid_selector?(path) ⇒ Boolean
document.valid_selector?(selector) → true/false
294 295 296 297 298 |
# File 'ext/yerba/yerba.c', line 294
static VALUE document_valid_selector_p(VALUE self, VALUE path) {
struct Document *document = get_document(self);
return yerba_document_valid_selector(document, StringValueCStr(path)) ? Qtrue : Qfalse;
}
|
#validate(schema, selector: nil) ⇒ Object
134 135 136 137 138 |
# File 'lib/yerba/document.rb', line 134 def validate(schema, selector: nil) schema_json = schema.is_a?(String) ? schema : JSON.generate(schema) validate_schema(schema_json, selector) end |
#validate_schema(*args) ⇒ Object
document.validate_schema(schema_json, selector = nil)
862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 |
# File 'ext/yerba/yerba.c', line 862
static VALUE document_validate_schema(int argc, VALUE *argv, VALUE self) {
VALUE schema_json, selector;
rb_scan_args(argc, argv, "11", &schema_json, &selector);
struct Document *document = get_document(self);
const char *selector_str = NIL_P(selector) ? NULL : StringValueCStr(selector);
char *result = yerba_document_validate_schema(document, StringValueCStr(schema_json), selector_str);
if (!result) return rb_ary_new();
VALUE json_string = make_utf8_string(result);
yerba_string_free(result);
return rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
}
|
#validate_selector!(selector) ⇒ Object
140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 |
# File 'lib/yerba/document.rb', line 140 def validate_selector!(selector) return if valid_selector?(selector) available = selectors = "selector \"#{selector}\" is not valid for this document" if available.any? suggestions = DidYouMean::SpellChecker.new(dictionary: available).correct(selector) if suggestions.any? += ". Did you mean: #{suggestions.first(3).join(", ")}?" else += ". Available selectors: #{available.first(5).join(", ")}" += ", ..." if available.length > 5 end end raise Yerba::SelectorNotFoundError, end |
#value_at(path) ⇒ Object
document.get_value(path) → parsed Ruby object (Hash/Array/String/Integer/etc)
301 302 303 304 305 306 307 308 309 310 311 |
# File 'ext/yerba/yerba.c', line 301
static VALUE document_get_value(VALUE self, VALUE path) {
struct Document *document = get_document(self);
char *json = yerba_document_get_value(document, StringValueCStr(path));
if (!json) return Qnil;
VALUE json_string = make_utf8_string(json);
yerba_string_free(json);
return rb_funcall(rb_path2class("JSON"), rb_intern("parse"), 1, json_string);
}
|
#where ⇒ Object
84 85 86 |
# File 'lib/yerba/document.rb', line 84 def where(...) root.where(...) end |
#write! ⇒ Object
document.save!
891 892 893 894 895 896 897 898 899 900 901 902 903 |
# File 'ext/yerba/yerba.c', line 891
static VALUE document_save(VALUE self) {
VALUE path = rb_iv_get(self, "@path");
if (NIL_P(path)) {
rb_raise(rb_eError, "Cannot save: document has no file path");
}
VALUE content = document_to_s(self);
rb_funcall(rb_cFile, rb_intern("write"), 2, path, content);
return self;
}
|